package com.kingtom.shengtai.api.controller;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.kingtom.kirin.core.common.base.PageInfo;
import com.kingtom.kirin.core.common.base.Result;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import com.kingtom.shengtai.api.mapper.SrmMapper;
import com.kingtom.shengtai.api.model.SrmSupplierDTO;
import com.kingtom.shengtai.app.srm.model.SrmSupplier;
import com.kingtom.shengtai.app.srm.model.SrmSupplierDO;
import com.kingtom.shengtai.app.srm.service.ISrmSupplierService;
import com.mybatisflex.core.paginate.Page;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * 供应商信息表 控制层。
 *
 * @author 86181
 * @since 2023-12-21
 */
@Tag(name = "SrmSupplierController", description = "供应商信息表 控制层。")
@RestController
@RequestMapping("/srmSuppliers")
public class SrmSupplierController{

    @Autowired
    private ISrmSupplierService iSrmSupplierService;

    /**
     * 添加供应商信息表。
     *
     * @param dto 供应商信息表
     * @return {@code true} 添加成功，{@code false} 添加失败
     */
    @Operation(summary = "添加供应商信息表。", description = "添加供应商信息表。")
    @PostMapping("/")
    public Result<SrmSupplierDTO> save(@RequestBody SrmSupplierDTO dto){
        SrmSupplier convert = SrmMapper.INSTANCE.convert(dto);
        SrmSupplierDO supplierDO = iSrmSupplierService.create(convert);
        return Result.ok(SrmSupplierDTO.convert(supplierDO));
    }

    /**
     * 根据主键删除供应商信息表。
     *
     * @param id 主键
     * @return {@code true} 删除成功，{@code false} 删除失败
     */
    @Parameter(name = "id", description = "主键", in = ParameterIn.PATH, required = true)
    @Operation(summary = "根据主键删除供应商信息表。", description = "根据主键删除供应商信息表。")
    @PostMapping("/{id}/")
    public Result<String> remove(@PathVariable String id){
        iSrmSupplierService.deleteById(id);
        return Result.ok(null);
    }

    /**
     * 根据主键删除供应商信息表。
     *
     * @param ids 主键
     * @return {@code true} 删除成功，{@code false} 删除失败
     */
    @Operation(summary = "根据主键批量删除供应商信息表。", description = "根据主键批量删除供应商信息表。")
    @PostMapping("/remove_batch")
    public Result<String> remove(@RequestBody List<String> ids){
        iSrmSupplierService.deleteByIds(ids);
        return Result.ok(null);
    }

    /**
     * 根据主键更新供应商信息表。
     *
     * @param dto 供应商信息表
     * @return {@code true} 更新成功，{@code false} 更新失败
     */
    @Parameter(name = "id", description = "", in = ParameterIn.PATH, required = true)
    @Operation(summary = "根据主键更新供应商信息表。", description = "根据主键更新供应商信息表。")
    @PostMapping("/{id}/info")
    public Result<SrmSupplierDTO> update(@RequestBody SrmSupplierDTO dto, @PathVariable String id){
        SrmSupplier convert = SrmMapper.INSTANCE.convert(dto);
        convert.setId(id);
        SrmSupplierDO supplierDO = iSrmSupplierService.update(convert);
        return Result.ok(SrmSupplierDTO.convert(supplierDO));
    }

    /**
     * 根据供应商信息表主键获取详细信息。
     *
     * @param id 供应商信息表主键
     * @return 供应商信息表详情
     */
    @Parameter(name = "id", description = "供应商信息表主键", in = ParameterIn.PATH, required = true)
    @Operation(summary = "根据供应商信息表主键获取详细信息。", description = "根据供应商信息表主键获取详细信息。")
    @GetMapping("/{id}/info")
    public Result<SrmSupplierDTO> getInfo(@PathVariable String id){
        SrmSupplierDO supplierDO = iSrmSupplierService.findById(id);
        return Result.ok(SrmSupplierDTO.convert(supplierDO));
    }

    /**
     * 分页查询供应商信息表。
     *
     * @param query    查询条件
     * @param pageNum  当前页码
     * @param pageSize 每页数量
     * @return 分页对象
     */
    @Operation(summary = "分页查询供应商信息表。", description = "分页查询供应商信息表。")
    @GetMapping("/page")
    public Result<PageInfo<SrmSupplierDTO>> page(String query, Integer pageNum, Integer pageSize){
        Map<String, Object> object = JsonUtils.toObject(query, Map.class);
        String name = object.get("name") instanceof String ? object.get("name").toString() : null;
        String type = object.get("type") instanceof String ? object.get("type").toString() : null;
        String state = object.get("state") instanceof String ? object.get("state").toString() : null;
        List<String> products = object.get("products") instanceof Collection<?> ?
                ((Collection<?>) object.get("products")).stream().map(e -> e instanceof String ? ((String) e) : null)
                        .collect(Collectors.toList()) : null;
        Page<SrmSupplierDO> page = iSrmSupplierService.page(name, type, state, products, pageNum, pageSize);
        return Result.ok(CollectionUtils.convertPage(page, SrmSupplierDTO::convert));
    }

}
